/*
  To access device, download LocalServer from https://stargate-system.com
  and run it on machine within the same network.

  Before uploading, replace <YOUR WIFI NAME> and <YOUR WIFI PASSWORD>
  with your WiFi credentials
*/

#include <WiFiGateDevice.h>

int triacPin = D5;
int thermometerPin = A0;

WiFiGateDevice device("My_WiFi_Name", "My_secret_password");

GateInt temperature = device.factory.createInt("output");
GateInt power = device.factory.createInt("input");

const int MEASUREMENT_INTERVAL = 1000;
unsigned long nextMeasurement = 0;
unsigned long powerTimer = 0;
bool triacOn = false;
unsigned int onTime = 0;
unsigned int offTime = 0;

void setup()
{
  pinMode(triacPin, OUTPUT);
  pinMode(thermometerPin, INPUT);

  device.setName("Power regulator");
  temperature.setName("Triac temp.");
  temperature.setMin(-20);
  temperature.setMax(125);
  power.setName("Power");
  power.setMax(100);
  power.setMin(0);
  power.setValue(0);
  power.onRemoteChange = onPowerChange;

  device.start();
}

void loop()
{
  device.loop();
  updatePower();
  if (nextMeasurement < millis()) {
    updateTemperature();
    nextMeasurement = millis() + MEASUREMENT_INTERVAL;
  }
}

void updateTemperature() {
  double voltage = analogRead(thermometerPin) * (3300.0 / 1024); // converting to mA
  const int zeroVoltage = 400;  // output at 0 centigrades (mV) acc. to datasheet
  const double temperatureCoefficient = 19.5; // output change per degree (mV) acc. to datasheet
  int currentTemperature = (int)((voltage - zeroVoltage) / temperatureCoefficient);
  temperature.setValue(currentTemperature);
}

void updatePower() {
  if ((power.getValue() > 0) && (temperature.getValue() < 100)) {
    if (triacOn) {
      if (millis() - powerTimer > onTime) {
        digitalWrite(triacPin, LOW);
        triacOn = false;
        powerTimer = millis();
      }
    } else {
      if (millis() - powerTimer > offTime) {
        digitalWrite(triacPin, HIGH);
        triacOn = true;
        powerTimer = millis();
      }
    }
  } else {
    digitalWrite(triacPin, LOW);
    triacOn = false;
  }
}

void onPowerChange() {
  const int timerStep = 50;
  onTime = power.getValue() * timerStep;
  offTime = (100 - power.getValue()) * timerStep;
}