const {GateDevice, Directions, ValueVisibility} = require('@stargate-system/device');

let layers = [];
const setLayers = () => {
    layers = [];
    for (let i = 0; i < distance.value + 1; i++) {
        layers.push(ambient.value);
    }
}

// Settings
const ambient = GateDevice.factory.createFloat(Directions.input);
ambient.valueName = 'Ambient';
ambient.setValue(20);
ambient.visibility = ValueVisibility.settings;
const cOut = GateDevice.factory.createFloat(Directions.input);
cOut.valueName = 'Out';
cOut.setValue(0.008);
cOut.visibility = ValueVisibility.settings;
const cTrans = GateDevice.factory.createFloat(Directions.input);
cTrans.valueName = 'Trans';
cTrans.setValue(0.08);
cTrans.visibility = ValueVisibility.settings;
const cIn = GateDevice.factory.createFloat(Directions.input);
cIn.valueName = 'In';
cIn.setValue(0.095);
cIn.visibility = ValueVisibility.settings;
const distance = GateDevice.factory.createInteger(Directions.input);
distance.valueName = 'Distance';
distance.setMinimum(0);
distance.setValue(5);
distance.visibility = ValueVisibility.settings;
distance.onRemoteUpdate = () => {
    setLayers();
}
const interval = GateDevice.factory.createFloat(Directions.input);
interval.valueName = 'Time compression';
interval.setRange([0.05, 1]);
interval.setValue(1);
interval.visibility = ValueVisibility.settings;

// Main
GateDevice.setName('Mock Heater');
const power = GateDevice.factory.createInteger(Directions.input);
power.valueName = 'Power';
power.setRange([0, 100]);
const temperature = GateDevice.factory.createInteger(Directions.output);
temperature.valueName = 'Temperature';
temperature.setValue(ambient.value);
const reset = GateDevice.factory.createBoolean(Directions.input);
reset.labelFalse = 'Reset';
reset.isButton = true;
reset.onRemoteUpdate = () => {
    if (reset.value) {
        for (let i = 0; i < layers.length; i++) {
            layers[i] = ambient.value;
        }
        temperature.setValue(ambient.value);
    }
}
interval.onRemoteUpdate = () => {
    clearInterval(intervalKey);
    intervalKey = setInterval(calculate, interval.value * 1000);
}
setLayers();
GateDevice.start();

const calculate = () => {
    for (let i = 0; i < layers.length; i++) {
        const qIn = i === 0 ? power.value * cIn.value : (layers[i - 1] - layers[i]) * cTrans.value;
        const qOut = i === layers.length - 1 ? (layers[i] - ambient.value) * cOut.value : (layers[i] - layers[i + 1]) * cTrans.value;
        layers[i] = layers[i] + (qIn - qOut);
    }
    temperature.setValue(layers[layers.length - 1]);
}

let intervalKey = setInterval(calculate, interval.value * 1000);
