const buffer = require('./dataBuffer');
const runUpRegulator = require('./runUpRegulator');
const stabilizingRegulator = require('./stabilizingRegulator');
const model = require('./model');
const config = require('../config');

const idleRegulator = {
    start: () => model.power.setValue(0),
    adjust: () => {}
}
let currentRegulator = idleRegulator;

const selectRegulator = () => {
    let selectedRegulator;
    if (model.temperature.value > model.setpoint.value + (config.stableRange)) {
        selectedRegulator = idleRegulator;
    } else if (model.temperature.value > model.setpoint.value - config.stableRange) {
        selectedRegulator = stabilizingRegulator;
    } else {
        selectedRegulator = runUpRegulator;
    }
    if (selectedRegulator !== currentRegulator) {
        selectedRegulator.start();
        currentRegulator = selectedRegulator;
    }
}

const init = () => {
    model.startSwitch.onRemoteUpdate = () => {
        if (model.startSwitch.value) {
            selectRegulator();
        } else {
            currentRegulator = idleRegulator;
            idleRegulator.start();
        }
    }
    model.setpoint.onRemoteUpdate = stabilizingRegulator.presetPower;
    setInterval(() => {
        buffer.add(model.temperature.value);
        if (model.startSwitch.value) {
            selectRegulator();
            currentRegulator.adjust();
        }
    }, config.measurementInterval * 1000 * config.timeCompression);
}

module.exports = {
    init
}