const model = require('./model');
const buffer = require('./dataBuffer');
const config = require('../config');

let lastSetpoint = 10;
let regulationInhibit = undefined;

const init = () => {
    model.power.setValue(lastSetpoint);
    currentLogic = operatingLogic;
    regulationInhibit = setTimeout(() => {
        regulationInhibit = undefined;
    }, config.reactionDelay * 1000 * config.timeCompression);
}

const setPower = (power) => {
    model.power.setValue(power);
    lastSetpoint = power;
    if (regulationInhibit) {
        clearTimeout(regulationInhibit);
    }
    regulationInhibit = setTimeout(() => {
        regulationInhibit = undefined;
    }, config.reactionDelay * 1000 * config.timeCompression);
}

const getDirection = (value) => {
    if (value === 0) {
        return 0;
    }

    return value > 0 ? 1 : -1;
}

const initialLogic = () => {
    if (buffer.getSlope() < -config.stableSlope) {
        init();
    } else if (buffer.getSlope() <= 0 && model.temperature.value < model.setpoint.value) {
        init();
    }
}

const operatingLogic = () => {
    if (!regulationInhibit) {
        const deviation = model.temperature.value - model.setpoint.value;
        if (Math.abs(deviation) > config.measurementAccuracy) {
            if (getDirection(buffer.getSlope()) === getDirection(deviation)) {
                const regulationValue = Math.max(1, (Math.abs(buffer.getSlope()) / config.stableSlope) * 0.2 * lastSetpoint);
                setPower(lastSetpoint - (getDirection(deviation) * regulationValue));
            }
        }
    }
}

let currentLogic = initialLogic;

const start = () => {
    model.power.setValue(0);
    if (regulationInhibit) {
        clearTimeout(regulationInhibit);
        regulationInhibit = undefined;
    }
    currentLogic = initialLogic;
    adjust();
}

const adjust = () => {
    currentLogic();
}

const presetPower = () => {
    lastSetpoint = (model.setpoint.value - config.ambientTemperature) * config.powerCoefficient;
}

module.exports = {
    start,
    adjust,
    presetPower
}